from .fetch import fetch_json
from typing import List


def get_templates() -> List[str]:
    repos = fetch_json("https://api.github.com/users/fsaccone/repos")

    repo_names: List[str] = list(
        map(
            lambda r: r["name"],
            repos,
        )
    )

    template_repo_names: List[str] = list(
        filter(
            lambda n: n.startswith("cli-template-"),
            repo_names,
        )
    )

    templates = list(
        map(
            lambda t: t.replace("cli-template-", "", 1),
            template_repo_names,
        )
    )

    return templates



from ..utils import logger


def _get_confirmation_input() -> bool:
    logger.log_important("(y/n) ", newline=False)
    confirmation = input().lower().strip()

    if not (confirmation == "y" or confirmation == "n"):
        return _get_confirmation_input()

    return True if confirmation == "y" else False


def confirmation(dir: str) -> bool:
    logger.log_information(
        "Are you sure you want to continue?",
        newline=False,
    )
    logger.log_important(f' All of the files in "{dir}" will be deleted.')

    return _get_confirmation_input()



from ..utils import logger
from typing import List


def unknown_template_message(template: str, templates: List[str]) -> None:
    logger.log_information(f'Unkown template "{template}"')
    logger.log_information("\nHere's a list of the templates:")
    logger.log_list(templates)



from ..utils import logger
from typing import List


def help_message(templates: List[str]) -> None:
    logger.log_information(
        'To build a project template, run "fsaccone --template <TEMPLATE>"'
    )
    logger.log_information("\nHere's a list of the templates:")
    logger.log_list(templates)



import argparse
import sys
from typing import Dict


def _parse_args() -> argparse.Namespace:
    args_parser = argparse.ArgumentParser()

    args_parser.add_argument(
        "--directory",
        "--dir",
        "-d",
        action="store",
        default=[None],
        nargs=1,
        type=str,
    )
    args_parser.add_argument(
        "--template",
        "-t",
        action="store",
        default=[None],
        nargs=1,
        type=str,
    )
    args_parser.add_argument(
        "-y",
        action="store_true",
        default=False,
    )

    args, unknown = args_parser.parse_known_args(sys.argv[1:])
    return args


def get_none_or_normalized_string(string: str | None) -> str | None:
    return string.lower().strip() if type(string) == str else None


def get_args() -> Dict:
    args = _parse_args()

    directory = get_none_or_normalized_string(args.directory[0])

    template = get_none_or_normalized_string(args.template[0])

    y: bool = args.y

    return {
        "directory": directory,
        "template": template,
        "y": y,
    }



from .utils.fetch import fetch_file
from .utils.logger import log_status, log_important
import os
import shutil
import sys
from zipfile import ZipFile


def clear_directory(dir: str) -> None:
    for child in os.listdir(dir):
        try:
            if os.path.isfile(child):
                os.unlink(child)
            elif os.path.isdir(child):
                shutil.rmtree(child)
        except Exception as e:
            log_important(f'Failed to delete "{child}". Reason: "{e}"')
            sys.exit(1)


def get_template_dir(root_dir: str) -> str:
    for child in os.listdir(root_dir):
        if child.startswith("fsaccone-cli-template-"):
            return child


def move_all_files(from_dir: str, to_dir: str) -> None:
    for child in os.listdir(from_dir):
        shutil.move(os.path.abspath(os.path.join(from_dir, child)), to_dir)


def build_template(template: str, dir: str) -> None:
    log_status("Deleting all the content in the directory...")
    clear_directory(dir)

    log_status("Downloading the template...")
    zip_url = f"https://github.com/fsaccone/cli-template-{template}/zipball/main"
    archive_path = os.path.abspath(os.path.join(dir, "template.zip"))
    fetch_file(zip_url, archive_path)

    log_status("Unzipping the archive...")
    zip_reference = ZipFile(archive_path, mode="r")
    zip_reference.extractall(dir)
    zip_reference.close()
    try:
        os.unlink(archive_path)
    except Exception as e:
        log_important(f'Failed to delete the archive. Reason: "{e}"')
        sys.exit(1)

    # Moving all the files in the extracted directory to the root directory.
    log_status("Building the template...")
    template_dir = get_template_dir(dir)
    move_all_files(template_dir, dir)
    try:
        shutil.rmtree(template_dir)
    except Exception as e:
        log_important(f'Failed to delete the unarchived directory. Reason: "{e}"')

    log_status("The template was built successfully.")



from .build import build_template
from .cli.confirm import confirmation
from .cli.help import help_message
from .cli.unknown import unknown_template_message
from .utils.templates import get_templates
import os


def start_app(args) -> None:
    templates = get_templates()

    template = args["template"]
    directory = os.path.abspath(args["directory"]) if args["directory"] else os.getcwd()

    if not os.path.exists(directory):
        os.makedirs(directory)

    if not template:
        help_message(templates)
        return

    if not template in templates:
        unknown_template_message(template, templates)
        return

    confirmed = True if args["y"] else confirmation(directory)

    if not confirmed:
        return

    build_template(template, directory)



import json
from typing import List, Dict
from urllib.request import urlopen, urlretrieve


def fetch_json(url: str) -> List | Dict:
    res: List = urlopen(url)
    data = json.load(res)

    return data


def fetch_file(url: str, path: str) -> None:
    urlretrieve(url, path)



from rich.console import Console
from typing import List

console = Console(color_system="standard")


def log_status(*text: List[str], newline=True) -> None:
    console.print(
        *text,
        style="cyan",
        end="\n" if newline else "",
    )


def log_important(*text: List[str], newline=True) -> None:
    console.print(
        *text,
        style="bold red",
        end="\n" if newline else "",
    )


def log_information(*text: List[str], newline=True) -> None:
    console.print(
        *text,
        style="italic bright_black",
        end="\n" if newline else "",
    )


def log_list(list: List[str], newline=True) -> None:
    for element in list:
        console.print(
            "  - ",
            style="bright_black",
            end="",
        )
        console.print(
            element,
            style="green",
            end="\n",
        )
